package org.jenkinsci.plugins.p4_client;

import hudson.FilePath.FileCallable;
import hudson.remoting.VirtualChannel;

import java.io.File;
import java.io.IOException;
import java.util.Map;

import org.jenkinsci.plugins.p4_client.client.ConnectionHelper;
import org.jenkinsci.plugins.p4_client.workspace.Workspace;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CheckoutTask implements FileCallable<Boolean> {

	private static final long serialVersionUID = 1L;
	private Logger logger = LoggerFactory.getLogger(CheckoutTask.class);

	private ConnectionHelper p4;

	private int change;
	private int review;
	private CheckoutStatus status;

	/**
	 * Constructor
	 * 
	 * @param config
	 *            - Server connection details
	 * @param auth
	 *            - Server login details
	 */
	public CheckoutTask(String credentialID, Workspace config) {
		try {
			p4 = new ConnectionHelper(credentialID);
			p4.login();
			p4.setClient(config);
		} catch (Exception e) {
			logger.error("Unable to connect.");
			e.printStackTrace();
		}
	}

	/**
	 * Invoke sync on build node (master or remote node).
	 * 
	 * @return true if updated, false if no change.
	 */
	public Boolean invoke(File workspace, VirtualChannel channel)
			throws IOException {
		logger.info("P4:CheckoutTask sync...");

		try {
			p4.tidyWorkspace();

			switch (status) {
			case HEAD:
				p4.syncFiles();
				break;

			case COMMITTED:
				p4.syncFiles(change);
				break;

			case SHELVED:
				p4.unshelveFiles(review);
				break;

			default:
				logger.error("P4:CheckoutTask unknown status: " + status);
				return false;
			}

			p4.disconnect();
		} catch (Exception e) {
			logger.error("Unable to sync workspace.");
			return false;
		}

		return true;
	}

	/**
	 * Takes the Build Variables set for the Jenkins Action and sets status,
	 * change and review numbers.
	 * 
	 * @param buildVariables
	 */
	public void setBuildVariables(Map<String, String> map) {
		if (map == null) {
			return;
		}
		
		if (map.containsKey("status")) {
			String value = map.get("status");
			status = CheckoutStatus.parse(value);
		}
		
		if (map.containsKey("change")) {
			String value = map.get("change");
			try {
				change = Integer.parseInt(value);
			} catch (NumberFormatException e) {
				change = 0;
			}
		}
		
		if (map.containsKey("review")) {
			String value = map.get("review");
			try {
				review = Integer.parseInt(value);
			} catch (NumberFormatException e) {
				change = 0;
			}
		}
	}

}
